class PrimitiveFigure {
  public :
    virtual ~PrimitiveFigure( void ) = 0;
    enum figKind { kPoint, kLine };
  protected : figKind kind;
  public : figKind getKind() { return kind; }
};
PrimitiveFigure::~PrimitiveFigure( void ) {}

class Figure {
  public :
    virtual bool hasMorePrimitive( void ) = 0;
    virtual PrimitiveFigure * nextPrimitive( void ) = 0;
    virtual ~Figure( void ) {};
    virtual Figure * clone() = 0;
};

class FigureBuilder {
  public :
    virtual void buildPoint( PrimitiveFigure & p ) {}
    virtual void buildLine( PrimitiveFigure & p ) {}
    virtual Figure * getResult( void ) = 0;
    virtual ~FigureBuilder( void ) {}
    virtual FigureBuilder * clone( void ) = 0;
};

class FigureConstructor {
  protected : FigureBuilder * builder;
  public :
    Figure * construct( Figure & fig ) {
      PrimitiveFigure * p;
      while( fig.hasMorePrimitive() ) {
        p = fig.nextPrimitive();
        switch( p->getKind() ) {
          case PrimitiveFigure::kPoint : builder->buildPoint( *p ); break;
          case PrimitiveFigure::kLine  : builder->buildLine( *p );  break;
          // 
        }
      }
      return builder->getResult();
    }
    void SetBuilder( FigureBuilder & b ) { delete builder; builder = &b; }
    FigureConstructor( FigureBuilder & b ) : builder( &b ) {}
    ~FigureConstructor( void ) { delete builder; }
    FigureConstructor & operator= ( FigureConstructor & cf ) {
      if( this != &cf ) {
        delete builder; builder = cf.builder->clone();
        // ɃRs[ׂ̂Rs[
      }
      return *this;
    }
};

class PostScriptFigure : public Figure {
  public : 
    PostScriptFigure( void ) {}
    PostScriptFigure( PostScriptFigure & psf ) {}
    PostScriptFigure * clone(void) {return new PostScriptFigure( *this );}
    bool hasMorePrimitive( void ) { /*  */ }
    PrimitiveFigure * nextPrimitive( void ) { /*  */ }
};
class PostScriptBuilder : public FigureBuilder {
  PostScriptFigure * result;
  public :
    void buildPoint( PrimitiveFigure & p ) { /*  */ }
    void buildLine( PrimitiveFigure & p ) { /*  */ }
    Figure * getResult( void ) { return result; }
    PostScriptBuilder( void ) {}
    PostScriptBuilder( PostScriptBuilder & psb )
    { result = psb.result->clone(); /* ɃRs[ׂ̂Rs[ */ }
    PostScriptBuilder * clone(void) {return new PostScriptBuilder( *this );}
};