abstract class PrimitiveFigure {
  public static final int kPoint = 0, kLine = 1;
  protected int kind;
  public int getKind() { return kind; }
}

interface Figure extends Cloneable {
  boolean hasMorePrimitive();
  PrimitiveFigure nextPrimitive();
  Object clone() throws CloneNotSupportedException;
}

abstract class FigureBuilder implements Cloneable {
  public void buildPoint( PrimitiveFigure p ) {}
  public void buildLine( PrimitiveFigure p ) {}
  abstract public Figure getResult();
  public Object clone() throws CloneNotSupportedException { 
    FigureBuilder instance = (FigureBuilder) super.clone();
      // 他にコピーすべきものもコピーする
    return instance;
  }
}

class FigureConstructor implements Cloneable {
  protected FigureBuilder builder;
  public Figure construct( Figure fig ) {
    PrimitiveFigure p;
    while( fig.hasMorePrimitive() ) {
      p = fig.nextPrimitive();
      switch( p.getKind() ) {
        case PrimitiveFigure.kPoint : builder.buildPoint( p ); break;
        case PrimitiveFigure.kLine  : builder.buildLine( p );  break;
        // 略
      }
    }
    return builder.getResult();
  }
  public void SetBuilder( FigureBuilder b ) { builder = b; }
  public FigureConstructor( FigureBuilder b ) { builder = b; }
  public Object clone() throws CloneNotSupportedException { 
    FigureConstructor instance = (FigureConstructor) super.clone();
    instance.builder = (FigureBuilder)builder.clone();
      // 他にコピーすべきものもコピーする
    return instance;
  }
}

class PostScriptFigure implements Figure {
  public boolean hasMorePrimitive() { /* 略 */ }
  public PrimitiveFigure nextPrimitive() { /* 略 */ }
  public Object clone() throws CloneNotSupportedException {
    PostScriptFigure instance = (PostScriptFigure) super.clone();
    return instance;
  }
}
class PostScriptBuilder extends FigureBuilder {
  PostScriptFigure result;
  public void buildPoint( PrimitiveFigure p ) { /* 略 */ }
  public void buildLine( PrimitiveFigure p ) { /* 略 */ }
  public Figure getResult() { return result; }
  public Object clone() throws CloneNotSupportedException { 
    PostScriptBuilder instance = (PostScriptBuilder) super.clone();
    instance.result = (PostScriptFigure) result.clone();
      // 他にコピーすべきものもコピーする
    return instance;
  }
}