#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

class Subject;
class Observer {
  public : 
    virtual ~Observer( void ) {}
    virtual void update( Subject * s, bool hasDestructed = false ) = 0;
};

class Subject {
    typedef vector< Observer* > ObsVector;
    typedef ObsVector::iterator ObsIterator;
  protected :
    ObsVector observers;
  public :
    virtual ~Subject( void ) = 0; // Subject𒊏ۃNXɂ
    virtual void attach( Observer * o ) {
      ObsIterator i = find( observers.begin(), observers.end(), o );
      if( i == observers.end() ) observers.push_back( o );
    }
    virtual void detach( Observer * o ) {
      ObsIterator i = find( observers.begin(), observers.end(), o );
      if( i != observers.end() ) observers.erase( i );
    }
    virtual void notify( bool hasDestructed = false ) {
      for( ObsIterator i = observers.begin(); i != observers.end(); i++ )
      { if( *i != 0 ) (*i)->update( this, hasDestructed ); }
    }
};
Subject::~Subject( void ) { notify( true ); }

class Data : public Subject {
    int value;
  public : 
    Data( void ) : value( 1 ) {}
    //ȉ̃Rs[֐łobserversNAĂ邱Ƃɒ
    Data( Data & d ) : Subject(), value( d.value ) { observers.clear(); } 
    Data & operator=( Data & d ) {
      if( this != &d )
      { Subject::operator=( d ); value = d.value; observers.clear(); }
      return *this;
    }
    void operation( void ) { ++value; notify(); }
    int getState( void ) { return value; }
};

class Graph : public Observer {
    Data * data;
  public :
    Graph( Data * d ) : data( d ) { d->attach( this ); }
    Graph( Graph & g ) : Observer( g ), data( g.data ) {}
    Graph & operator=( Graph & g ) {
      if( this != &g ) {
        Observer::operator=( g ); 
        data->detach( this ); // O̓o^
        data = g.data; data->attach( this ); // o^̍XV
      }
      return *this;
    }
    ~Graph( void ) { if( data != 0 )  data->detach( this ); }
    void update( Subject * s, bool hasDestructed = false ) {
      if( hasDestructed && (s == data) ) data = 0;
      if( data != 0 && (s == data) )
        for( int i = 0; i < data->getState(); i++ ) cout << "*";
      cout << endl; 
    }
};

int main( void )
{
  Data * d1 = new Data(); Data * d2 = new Data(); d2->operation(); 
  Graph * g1 = new Graph( d1 ); Graph * g2 = new Graph( d2 );
  d1->operation(); d2->operation(); 
  return 0;
}