import java.util.*; // for Java2 (JDK 1.2.x)

// 例外の定義
class CompositeException extends Exception {
  public CompositeException() { super(); }
}
class NonCompositeException extends CompositeException {
  public NonCompositeException() { super(); }
}
class ElementNotFoundException extends CompositeException {
  public ElementNotFoundException() { super(); }
}
class NotAddedException extends CompositeException {
  private Component notAddedComponent;
  public NotAddedException( Component c ) {
    super(); notAddedComponent = c;
  }
  public Component getNotAdded() { return notAddedComponent; }
}

abstract class Component {
  public abstract void operation();
  public void add( Component c )
    throws NonCompositeException, NotAddedException {
    throw new NotAddedException( c );
  }
  public void remove( Component c ) 
    throws NonCompositeException, ElementNotFoundException {
    throw new NonCompositeException();
  }
  public Iterator getChild() throws NonCompositeException { 
    throw new NonCompositeException();
  }
}

class Composite extends Component {
  private Vector children = new Vector();
  public void operation() {
    Iterator i = children.iterator();
    while( i.hasNext() ) ((Component)(i.next())).operation();
  }
  public void add( Component c )
    throws NonCompositeException, NotAddedException {
    if( children.add( c ) == false ) throw new NotAddedException( c );
  }
  public void remove( Component c ) 
    throws NonCompositeException, ElementNotFoundException {
    if( children.remove( c ) == false ) 
      throw new ElementNotFoundException();
  }
  public Iterator getChild() throws NonCompositeException {
    return children.iterator();
  }
}

class Leaf extends Component {
  private Object value;
  public Leaf( Object v ) { value = v; }
  public void operation() { System.out.println( value ); }
}

public class Composite1 {
  public static void main( String args[] ) {
    Component node1 = new Composite(), node2 = new Composite(),
              node3 = new Composite();
    Component node4 = new Leaf( new Integer( 1 ) ), 
              node5 = new Leaf( new Integer( 2 ) ),
              node6 = new Leaf( new Integer( 3 ) ),
              node7 = new Leaf( new Integer( 4 ) );
    // 以下がユーザのコード部分
    try {
      node2.add( node4 ); node2.add( node5 ); node3.add( node6 );
      node3.add( node7 ); node1.add( node2 ); node1.add( node3 );
      node1.operation();  System.out.println();
      node1.remove( node3 ); node1.operation();
    }
    catch( NotAddedException e ) {
      // 要素が追加できなかった
      Component c = e.getNotAdded(); // 追加できなかった要素を取り戻す
    }
    catch( NonCompositeException e )
    { /* 非Compositeオブジェクトに違法なメソッド呼び出しをした */ }
    catch( ElementNotFoundException e )
    { /* 指定された要素が見つからなかった */ }
    catch( CompositeException e ) { /* その他の例外に対する処理 */ }
  }
}