#include <iostream>
#include <string>
#include <vector>
using namespace std;

class Directory; class File;

class Visitor {
  public :
    virtual void visitDirectory( Directory * d ) = 0;
    virtual void visitFile( File * f ) = 0;
};

class DiskElement { // fBNgKw؍\̗vf
  protected :
    string name; DiskElement * parent;
  public : 
    DiskElement( const string & name ) : name( name ), parent( 0 ) {}
    virtual void accept( Visitor * v ) = 0;
    string getName( void ) { return name; }
    DiskElement * getParent( void ) { return parent; }
    void setParent( DiskElement * p ) { parent = p; }
};
typedef vector<DiskElement *>::iterator iter;

class Directory : public DiskElement { // fBNg
    vector<DiskElement *> children;
  public :
    Directory( const string & name ) : DiskElement( name ) {}
    void accept( Visitor * v ) {
      v->visitDirectory( this );
      for( iter i = children.begin(); i != children.end(); i++ )
        (*i)->accept( v );
    }
    void add( DiskElement * e )
    { children.push_back( e ); e->setParent( this ); }
};

class File : public DiskElement { // t@C
    unsigned long size; // t@CTCY
  public :
    File( const string & name, unsigned long size )
      : DiskElement( name ), size( size ) {}
    void accept( Visitor * v ) { v->visitFile( this ); }
    unsigned long getSize( void ) { return size; }
};

class GettingSizeVisitor : public Visitor { // 헪1FTCY擾
    unsigned long size;
  public :
    GettingSizeVisitor( void ) : size( 0 ) {}
    void visitDirectory( Directory * d ) {}
    void visitFile( File * f ) { size += f->getSize(); }
    unsigned long getSize( void )
    { unsigned long s = size; size = 0; return s; }
};

class ListingVisitor : public Visitor { // 헪2FXg\
    void printSpace( int j ) { for( int i = 0; i < j; i++ ) cout << " "; }
    int getLevel( DiskElement * d ) {
      int i = 0; DiskElement * t = d;
      while( t->getParent() != 0 ) { t = t->getParent(); i++; }
      return i;
    }
  public :
    void visitDirectory( Directory * d )
    { printSpace( getLevel(d) * 3 ); cout << d->getName() << endl; }
    void visitFile( File * f )
    { printSpace( getLevel(f) * 3 ); cout << f->getName() << endl; }
};

int main( void ) {
  File * node4 = new File( "node4", 1 ), * node5 = new File( "node5", 2 ), 
       * node6 = new File( "node6", 3 ), * node7 = new File( "node7", 4 );
  Directory * node2 = new Directory( "node2" ),
            * node3 = new Directory( "node3" ),
            * node1 = new Directory( "node1" );
  node2->add( node4 ); node2->add( node5 ); node3->add( node6 ); 
  node3->add( node7 ); node1->add( node2 ); node1->add( node3 );

  GettingSizeVisitor gsVisitor; ListingVisitor lVisitor;
  node1->accept( &gsVisitor );
  cout << "used disk size:" << gsVisitor.getSize() << endl;
  node1->accept( &lVisitor );
  return 0;
}