import java.util.*; // for Java 2 (JDK1.2.x)

interface Visitor {
  void visitDirectory( Directory d );
  void visitFile( File f );
}

abstract class DiskElement { // ディレクトリ階層木構造の要素
  protected String name; protected DiskElement parent;
  public DiskElement( String name ) { this.name = name; parent = null; }
  abstract public void accept( Visitor v );
  public final String getName() { return name; }
  public final DiskElement getParent() { return parent; }
  public final void setParent( DiskElement p ) { parent = p; }
}

class Directory extends DiskElement { // ディレクトリ
  private Vector children = new Vector();
  public Directory( String name ) { super( name ); }
  public void accept( Visitor v ) {
    v.visitDirectory( this );
    Iterator i = children.iterator();
      while( i.hasNext() ) ((DiskElement)(i.next())).accept( v );
  }
  public final void add( DiskElement e )
  { children.add( e ); e.setParent( this ); }
}

class File extends DiskElement { // ファイル
  private long size; // ファイルサイズ
  public File( String name, long size )
  { super( name ); this.size = size; }
  public void accept( Visitor v ) { v.visitFile( this ); }
  public final long getSize() { return size; }
}

class GettingSizeVisitor implements Visitor { // 戦略1：サイズ取得
  private long size;
  public GettingSizeVisitor() { size = 0; }
  public void visitDirectory( Directory d ) {}
  public void visitFile( File f ) { size += f.getSize(); }
  public long getSize() { long s = size; size = 0; return s; }
}

class ListingVisitor implements Visitor { // 戦略2：リスト表示
  private void printSpace( int j )
  { for( int i = 0; i < j; i++ ) System.out.print(" "); }
  private int getLevel( DiskElement d ) {
    int i = 0; DiskElement t = d;
    while( t.getParent() != null ) { t = t.getParent(); i++; }
    return i;
  }
  public void visitDirectory( Directory d )
  { printSpace( getLevel(d) * 3 ); System.out.println( d.getName() ); }
  public void visitFile( File f )
  { printSpace( getLevel(f) * 3 ); System.out.println( f.getName() ); }
}

public class Visitor1 {
	public static void main( String args[] ) {
    File node4 = new File( "node4", 1 ), node5 = new File( "node5", 2 ), 
         node6 = new File( "node6", 3 ), node7 = new File( "node7", 4 );
    Directory node2 = new Directory( "node2" ),
              node3 = new Directory( "node3" ),
              node1 = new Directory( "node1" );
    node2.add( node4 ); node2.add( node5 ); node3.add( node6 ); 
    node3.add( node7 ); node1.add( node2 ); node1.add( node3 );
  
    GettingSizeVisitor gsVisitor = new GettingSizeVisitor();
    ListingVisitor lVisitor = new ListingVisitor();
    node1.accept( gsVisitor );
    
    System.out.println( "used disk size:" + gsVisitor.getSize() );
    node1.accept( lVisitor );
  }
}
