#include <vector>
#include <string>
#include <iostream>
using namespace std;

class Command { // CommandC^tFCX
  public :
    virtual ~Command( void ) {}
    virtual Command * clone( void ) = 0;
    virtual void execute( bool redo ) = 0;//redoɂ́gĎshtruen
    virtual void unexecute( void ) = 0;
};

class MenuItem { // Invokerɑ
    Command * command;
  public :
    MenuItem( void ) : command( 0 ) {}
    void setCommand( Command * c ) { command = c; }
    virtual void invoke( void ) {
      if( command != 0 ) command->execute( false );
    }
};

class CommandList { // ߗXg
    vector<Command*> commands;
    vector<Command*>::iterator itor;
  public :
    CommandList( void ) : commands(), itor( commands.end() ) {}
    void pushCommand( Command * c ) {
      vector<Command*>::iterator i = itor;
      while( itor != commands.end() ) {
        delete commands.back(); commands.pop_back();
      }
      Command * clone = c->clone();
      if( clone != 0 ) commands.push_back( clone );
      itor = commands.end();
    }
    void undoAction( void ) { // gh̃ANV
      if( itor == commands.begin() ) {
        cout << "can't undo" << endl; return;
      }
      (*(--itor))->unexecute();
    }
    void redoAction( void ) { // gĎsh̃ANV
      if( itor == commands.end() ) {
        cout << "can't redo" << endl; return;
      }
      (*(itor++))->execute( true );
    }
};

class TextDocument { // Receiverɑ
    string state;           // f[^ // 
    size_t insertionpoint;  // }|Cg // 
  public :
    CommandList clist; // ߗXghLgɎ
    TextDocument( void ) : state( "012345" ), insertionpoint( 0 ) {}
    string & getData( void ) { return state; }
    void setData( string & str ) { state = str; }
    void setInsertionPoint( size_t i ) { insertionpoint = i; }
    size_t getInsertionPoint( void ) { return insertionpoint; }
    void insertAction( string & s ) { state.insert( insertionpoint, s ); }
    void removeAction( string::size_type pos, string::size_type len ) {
      state.erase( pos, len );
    }
};

class Application { // Clientɑ
    TextDocument * document;
    MenuItem * insertMenuItem, * removeLastCharMenuItem,
             * undoMenuItem, * redoMenuItem;
  public :
    Application( void );
    TextDocument * getCurrentDocument( void );
    void print( void );
    void test( void );
};

// eLXgɕ"abc"}閽
class InsertCommand : public Command {
    Application * appl; TextDocument * document;
    size_t insertionpoint; string inserted_string;
  public :
    InsertCommand( Application * a ) : appl( a ) {}
    InsertCommand * clone( void ) { return new InsertCommand( *this ); }
    void execute( bool redo ) {
      string s = "abc";
      if( redo ) document->setInsertionPoint( insertionpoint );
      (document = appl->getCurrentDocument())->insertAction( s );
      insertionpoint = document->getInsertionPoint();
      inserted_string = s; 
      if( ! redo ) document->clist.pushCommand( this );
    }
    void unexecute( void ) {
      document->removeAction( insertionpoint, inserted_string.size() );
      document->setInsertionPoint( insertionpoint );
    }
};

// eLXg̖1폜閽
class RemoveLastCharCommand : public Command {
    Application * appl; TextDocument * document; char deleted_char;
  public :
    RemoveLastCharCommand( Application * a ) : appl( a ) {}
    RemoveLastCharCommand * clone( void ) {
      return new RemoveLastCharCommand( *this );
    }
    void execute( bool redo ) {
      document = appl->getCurrentDocument();
      deleted_char =  document->getData()[ document->getData().size() - 1 ];
      document->removeAction( document->getData().size() - 1, 1 );
      if( ! redo ) document->clist.pushCommand( this );
    }
    void unexecute( void ) {
      size_t insertionpoint = document->getInsertionPoint();
      document->setInsertionPoint( document->getData().size() );
      document->insertAction( string( 1, deleted_char ) );
      document->setInsertionPoint( insertionpoint );
    }
};

// gh
class UndoCommand : public Command {
    Application * appl; TextDocument * document;
  public :
    UndoCommand( Application * a ) : appl( a ) {}
    UndoCommand * clone( void ) { return 0; }
    void execute( bool redo ) {
      document = appl->getCurrentDocument(); document->clist.undoAction();
    }
    void unexecute( void ) {}
};

// gĎsh
class RedoCommand : public Command {
    Application * appl; TextDocument * document;
  public :
    RedoCommand( Application * a ) : appl( a ) {}
    RedoCommand * clone( void ) { return 0; }
    void execute( bool redo ) {
      document = appl->getCurrentDocument(); document->clist.redoAction();
    }
    void unexecute( void ) {}
};

Application::Application( void ) { // ApplicatioñRXgN^
  document = new TextDocument();
  insertMenuItem = new MenuItem(); removeLastCharMenuItem = new MenuItem(); 
  undoMenuItem   = new MenuItem(); redoMenuItem = new MenuItem();
  insertMenuItem->setCommand( new InsertCommand( this ) );
  removeLastCharMenuItem->setCommand( new RemoveLastCharCommand( this ) );
  undoMenuItem->setCommand( new UndoCommand( this ) );
  redoMenuItem->setCommand( new RedoCommand( this ) );
}
// ݂̃hLg𓾂郁\bh
TextDocument * Application::getCurrentDocument( void ) { return document; }
// hLg̓e\郁\bh
void Application::print( void ) { cout << document->getData() << endl; }

void Application::test( void ) { // eXgp\bh
  print();
  document->setInsertionPoint( 2 ); insertMenuItem->invoke(); print();
  document->setInsertionPoint( 4 ); insertMenuItem->invoke(); print();
  removeLastCharMenuItem->invoke(); print();
  cout << "------------" << endl;
  undoMenuItem->invoke(); print(); undoMenuItem->invoke(); print();
  undoMenuItem->invoke(); print();
  cout << "------------" << endl;
  redoMenuItem->invoke(); print(); redoMenuItem->invoke(); print();
  cout << "------------" << endl;
  document->setInsertionPoint( 1 ); insertMenuItem->invoke(); print();
  undoMenuItem->invoke(); print(); redoMenuItem->invoke(); print();
  redoMenuItem->invoke(); print();
}

int main( void ) {
  Application appl; appl.test();
  return 0;
}